"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStandaloneRtoModule = exports.generateRtoModules = exports.parseTypeOnly = void 0;
const parse_typeonly_1 = require("./parser/parse-typeonly");
const ProjectInputOutput_1 = require("./rto-factory/ProjectInputOutput");
const RtoModuleFactory_1 = require("./rto-factory/RtoModuleFactory");
const RtoProject_1 = require("./rto-factory/RtoProject");
function parseTypeOnly(options) {
    const ast = parse_typeonly_1.parseTypeOnlyToAst(options.source);
    if (options.return && options.return.freeze)
        deepFreezePojo(ast);
    return ast;
}
exports.parseTypeOnly = parseTypeOnly;
async function generateRtoModules(options) {
    let astProvider = options.astProvider;
    if (!astProvider) {
        if (!options.readFiles)
            throw new Error(`A parameter 'readFiles' or 'astProvider' is required.`);
        astProvider = ProjectInputOutput_1.makeReadSourceFileAstProvider(options.readFiles.sourceDir, options.readFiles.encoding || "utf8");
    }
    let wfOpt2;
    if (options.writeFiles) {
        const wfOpt1 = options.writeFiles === true ? {} : options.writeFiles;
        const encoding = wfOpt1.encoding || "utf8";
        const outputDir = wfOpt1.outputDir || (!options.astProvider && options.readFiles && options.readFiles.sourceDir);
        if (!outputDir)
            throw new Error(`Option 'writeFiles.outputDir' is required when 'readFiles.sourceDir' is not provided.`);
        wfOpt2 = {
            encoding,
            outputDir,
            prettify: wfOpt1.prettify
        };
    }
    const output = new ProjectInputOutput_1.RtoProjectOutput({
        returnRtoModules: !!options.returnRtoModules,
        writeFiles: wfOpt2
    });
    const project = new RtoProject_1.default({
        astProvider,
        rtoModuleListener: output.listener,
        moduleFactoryOptions: {
            defineGlobals: options.defineGlobals
        }
    });
    await project.addModules(options.modulePaths);
    if (options.returnRtoModules) {
        const rtoModules = output.getRtoModules();
        if (options.returnRtoModules !== true && options.returnRtoModules.freeze)
            deepFreezePojo(rtoModules);
        return rtoModules;
    }
}
exports.generateRtoModules = generateRtoModules;
function createStandaloneRtoModule(options) {
    const factory = new RtoModuleFactory_1.default(options.ast, undefined, { defineGlobals: options.defineGlobals });
    const rtoModule = factory.createRtoModule();
    if (options.freeze)
        deepFreezePojo(rtoModule);
    return rtoModule;
}
exports.createStandaloneRtoModule = createStandaloneRtoModule;
function deepFreezePojo(object) {
    if (Object.isFrozen(object))
        return object;
    Object.freeze(object);
    for (const key of Object.keys(object)) {
        const value = object[key];
        if (value && typeof value === "object")
            deepFreezePojo(value);
    }
    return object;
}
