"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const module_path_helpers_1 = require("../helpers/module-path-helpers");
const RtoModuleFactory_1 = require("./RtoModuleFactory");
class RtoProject {
    constructor(options) {
        this.options = options;
        this.factories = new Map();
    }
    async addModules(modulePaths) {
        for (const from of modulePaths)
            await this.importModule({ from });
        for (const factory of this.factories.values())
            await factory.loadImports(modulePath => this.importModule(modulePath));
        for (const factory of this.factories.values()) {
            const module = factory.createRtoModule();
            await this.options.rtoModuleListener(module, factory.getModulePath());
        }
    }
    async importModule(relPath) {
        const modulePath = module_path_helpers_1.toModulePath(Object.assign(Object.assign({}, relPath), { removeExtensions: [".d.ts", ".ts"] }));
        let factory = this.factories.get(modulePath);
        if (!factory) {
            const ast = await this.options.astProvider(modulePath);
            factory = new RtoModuleFactory_1.default(ast, modulePath, this.options.moduleFactoryOptions);
            this.factories.set(modulePath, factory);
        }
        return factory;
    }
}
exports.default = RtoProject;
